----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:19:43 09/13/2010 
-- Design Name: 
-- Module Name:    Dynamic_Bandwidth_Allocator - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.STD_LOGIC_ARITH.ALL;
--use IEEE.STD_LOGIC_unSIGNED.ALL;
use IEEE.numeric_std.all; 

library unisim;
use unisim.vcomponents.all;


-- Instanciation Template:
--Component Dynamic_Bandwidth_Allocator
--   generic (bandwidth : integer :=10); -- the bit width of the bandwidth
--   port (clk,Reset,DBA_CS : in std_logic;
--         Channel_Select : out std_logic_vector (1 downto 0);
--         Bandwidth_00,
--         Bandwidth_01,
--         Bandwidth_10,
--         Bandwidth_11 : inout unsigned (bandwidth+1 downto 0)
--         );
--   end component;
--DBA: Dynamic_Bandwidth_Allocator
--   generic map(10);
--   port map(clk => CLK_50M,
--            Channel_Select,threshold_1,threshold_2,threshold_3);

entity Deterministic_Channel_Selector is
   generic (bandwidth :integer:=10-- the bit width of the bandwidth
            );
   port (clk,Reset,DBA_CS : in std_logic;
         Channel_Select : out std_logic_vector (1 downto 0);
         Bandwidth_00,
         Bandwidth_01,
         Bandwidth_10,
         Bandwidth_11 : in unsigned (bandwidth+1 downto 0)---------------------------------------
         );
   
   signal   Bandwidth_Counter_1,
            Bandwidth_Counter_2,
            Bandwidth_Counter_3,
            Bandwidth_Counter_4 :   unsigned(bandwidth downto 0):=(OTHERS=>'0');  --This extra bit work as the carrying bit.
   signal   Channel_Bandwidth_00 : unsigned (bandwidth+1 downto 0):= "00" & to_unsigned(692,10); 
   signal   Channel_Bandwidth_01 : unsigned (bandwidth+1 downto 0):= "01" & to_unsigned(  0,10); 
   signal   Channel_Bandwidth_10 : unsigned (bandwidth+1 downto 0):= "10" & to_unsigned( 26,10); 
   signal   Channel_Bandwidth_11 : unsigned (bandwidth+1 downto 0):= "11" & to_unsigned(305,10);  
   
end Deterministic_Channel_Selector;

architecture Behavioral of Deterministic_Channel_Selector is
begin
--****************************************************************************
--                           begining of Behavior
--****************************************************************************

Channel_Selector:process(CLK)
   begin         
      if rising_edge(clk) then
         IF DBA_CS = '1' then  --DBA_CS is the slection pin for working. 
            if Bandwidth_Counter_4(bandwidth)='0' then  --new bandwidth information is read when BDA_CS= '0'
               Channel_Bandwidth_00<= Channel_Bandwidth_01;
               Channel_Bandwidth_01<= Channel_Bandwidth_10;
               Channel_Bandwidth_10<= Channel_Bandwidth_11;
               Channel_Bandwidth_11<= Channel_Bandwidth_00;
               Bandwidth_Counter_1 <= Bandwidth_Counter_2;
               Bandwidth_Counter_2 <= Bandwidth_Counter_3;
               Bandwidth_Counter_3 <= Bandwidth_Counter_4;
               Bandwidth_Counter_4 <= Bandwidth_Counter_1 + Unsigned('0' & Channel_Bandwidth_00(bandwidth-1 downto 0));
            else   
               if Reset='1' then --Synchronous Reset Design
                  Channel_Select<=std_logic_vector(Channel_Bandwidth_11(bandwidth+1 downto bandwidth)); 
                  Bandwidth_Counter_4(bandwidth)<='0';
               end if;
            end if;
         else 
            Channel_Bandwidth_00<=Bandwidth_00;
            Channel_Bandwidth_01<=Bandwidth_01;
            Channel_Bandwidth_10<=Bandwidth_10;
            Channel_Bandwidth_11<=Bandwidth_11;
--            Bandwidth_Counter_1 <= to_unsigned(0,bandwidth+2);--Do not have to be 0123. can be 0000.
--            Bandwidth_Counter_2 <= to_unsigned(1,bandwidth+2);
--            Bandwidth_Counter_3 <= to_unsigned(2,bandwidth+2);
--            Bandwidth_Counter_4 <= to_unsigned(3,bandwidth+2);
         end if;
      end if;
   end process;
end Behavioral;